unit mausb;
{      Alle Mausfunktionen werden ber den Interrupt 33H angesteuert. Die
       Parameterbergabe erfolgt in den CPU-Registern. AX enthlt die
       Nummer der auszufhrenden Funktion. Sind weitere Parameter ntig,
       werden sie in bx, cx, dx, si, di und es bergeben. Alle Register,
       die keine Rckgabewerte enthalten, bleiben whrend des Interrupts
       erhalten. Es gibt folgende Funktionen:

       Funktion    Bedeutung
       -------------------------------------------------------------
       AX = 0  Initialisierung der Maus
               Rckgabe: AX = -1 (Maus ist vorhanden)
                         BX = 3 (Zahl der Tasten)

       AX = 1  Mauscursor sichtbar schalten

       AX = 2  Mauscursor unsichtbar schalten

       AX = 3  Tastenstatus und Position auslesen
               Rckgabe: BX Tastenstatus
                          (Bits 0/1/2: links/rechts/mitte,1=gedrckt)
                       CX x-position
                       DX y-position

       AX = 4  Mausposition setzen
               CX = neue x-Position
               DX = neue y-Position (Angaben immer in Pixeln, auch
                                       in den Textmodi!)

       AX = 5  Auslesen der 'Button Press Info'
               BX = Nummer der Taste (0=links, 1=rechts, 2=mitte)
               Rckgabe: BX = Anzahl der Tastendrcke
                       CX = x-Position des letzten Drckens
                       DX = y-Position des letzten Drckens

       AX = 6  Auslesen der 'Button Release Info'
               BX = Nummer der Taste (0=links, 1=rechts, 2=mitte)
               Rckgabe: BX = Anzahl des Tastenlsens
                       CX = x-Position des letzten Lsens
                       DX = y-Position des letzten Lsens

       AX = 7  Minimale/Maximale x-Position setzen
               CX = Minimum
               DX = Maximum (Der Mauscursor bewegt sich nur zwischen
                               diesen Werten)

       AX = 8  Minimale/Maximale y-Position setzen
               CX = Minimum
               DX = Maximum

       AX = 9  Grafischen Cursor definieren
               BX = relative x-Position
               CX = relative y-Position
               ES:DX = Zeiger auf neuen Cursorblock
                   (Zunchst 16 Worte mit Cursormaske, dann
                    16 Worte mit den Cursorwerten)

       AX = 10 Text Cursor definieren
               BX = 0: Software Cursor, 1: Hardware Cursor
               CX = Software: Bildschirmmaske,Hardware: scan line start
               DX = Software: Bildschirmwert, Hardware: scan line end

       AX = 11 Bewegungswerte lesen
               Rckgabe: CX = akkumulierter x-Zhler
                       DX = akkumulierter y-Zhler
               Gezhlt werden 'Mickeys' (1/200 Zoll). Nach Aufruf
               werden die Zhler auf Null gesetzt

       AX = 12 Interrupt Maske und Einsprung definieren
               CX = Maske
               ES:DX = Interrupt Einsprung
               Die Interruptroutine muss mit 'RETF' (nicht IRET!)
               abgeschlossen sein. Maske:
               Bit 0:  1:Cursor hat sich bewegt
                   1:  1:linke Taste gedrckt
                   2:  1:linke Taste gelst
                   3:  1:rechte Taste gedrckt
                   4:  1:rechte Taste gelst
                   5:  1:mittlere Taste gedrckt
                   6:  1:mittlere Taste gelst
               Ist das entsprechende Bit in der Maske 1, wird die
               Routine beim Auftreten der Bedingung angefahren.
               Beim Einsprung in die Routine haben die Register

               folgende Belegung:
               AX: Bits, warum Routine gestartet wird, Belegung s.o.
               BX: Tastenstatus
               CX: x-Position
               DX: y-Position
               SI: akkumulierter x-Zhler
               DI: akkumulierter y-Zhler
               (Hinweis: zum Abschalten der Interruptroutine, Funktion
               12 mit CX = 0 aufrufen)

       AX = 13 Light-Pen Emulation ein
               Die Funktionen des Lichtgriffels werden auf die Maus
               abgebildet

       AX = 14 Light-Pen Emulation aus
               Die Emulation wird wieder ausgeschaltet

       AX = 15 Mickey/Pixel Rate setzen
               CX = x-Rate (n Mickeys pro 8 Pixel in x-Richtung)
               DX = y-Rate (n Mickeys pro 8 Pixel in y-Richtung)
               Legt fest, wie schnell die Maus ber den Bildschirm
               bewegt wird.

       AX = 16 Bedingtes Ausschalten
               CX = linke Seite
               DX = obere Seite
               SI = rechte Seite
               DI = untere Seite (alles in Pixeln)
               Sobald der Cursor in diesen Bereich bewegt wird, wird er
               ausgeschaltet. Ntzlich zum Bildaufbau.  Wieder
               Einschalten durch Funktion 1.

       AX = 19 Doppelte Geschwindigkeit setzen
               DX = Mickeys/Sekunde
               wird diese Geschwindigkeit berschritten, bewegt sich
               der Cursor doppelt so schnell ber den Bildschirm
               Zum Ausschalten: Sehr grossen Wert in CX bergeben

       AX = 20 Interrupt Routine austauschen
               BX: Segment der neuen Routine
               CX: neue Maske
               DX: Offset der neuen Routine (siehe Funktion 9)

               Rckgabe: BX: Segment der alten Routine
                       CX: alte Maske
                       DX: Offset der alten Routine

       AX = 29 Seitennummer des Bildschirms fr Cursor setzen
               BX = neue Seite
               Wenn der Bildschirmadapter mehr als eine Seite speichern
               kann, wird hier bestimmt, auf welcher Seite der Maus-
               cursor erscheint.

       AX = 30 Seitennummer auslesen
               Rckgabe: BX = Seitennummer fr Mauscursor

       Achtung!!

       Es werden keinerlei Prfungen der Register vorgenommen, falsche
       Werte knnen das Programm zu unvorhersagbaren Reaktionen
       veranlassen.}
{
        Anmerkung: Der normale Grafikcursor funktioniert bei der
        Hercules-Karte NICHT. (Siehe Dos-Heft, Ausgabe November
        1988, Seite 20; Pascal August/September 1988, Seite 10.)
        Dem Maustreiber mu nach dem umschalten in den Grafikmodus
        erst mitgeteilt werden, da sich der Rechner im Hercules-
        Grafikmodus befindet. Dies geschieht unter Turbo-Pascal
        mit "mem[$40:$49] := 6;" fr die Grafikseite 0 und mit
        "mem [$40:$49] := 5" fr die Grafikseite 1. Auerdem mu
        ein nochmaliger Maus-Reset durchgefhrt werden. Die hier
        aufgefhrte Prozedur MAUSGRAFIKHERCULES verwirklicht diese
        Forderungen (fr Grafikseite 0). Es ist seitens des Anwen-
        ders dafr zu sorgen, da der Grafikmodus mit "closegraph"
        wieder verlassen wird.
        (Versucht man es trotzdem mit der Prozedur "Mausgrafik-
        cursor", so zeigen sich lediglich "wandernde Punkte"
        anstatt eines Mauspfeiles)

  !!!

}

interface

          type shape = record
                         screen,
                         mask    : array[0..15] of word;
                       end;

Const standardshape :
      shape = (screen : ($ffff,$ffff,$ffff,$ffff,
                         $ffff,$ffff,$ffff,$ffff,
                         $ffff,$ffff,$ffff,$ffff,
                         $ffff,$ffff,$ffff,$ffff);
               mask   : ($8000,$e000,$f800,$fe00,
                         $d800,$0c00,$0600,$0300,
                         $0000,$0000,$0000,$0000,
                         $0000,$0000,$0000,$0000));
Const Grospfeil :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($FFE0,$FFC0,$FF80,$FF00,
                         $FF00,$FF00,$FF00,$E380,
                         $C1C0,$00E0,$0070,$0038,
                         $001C,$000E,$0007,$0003));
Const haus :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FECF,$FD4F,$FB8F,$F7CF,
                         $EFEF,$DFF7,$8003,$5FF5,
                         $D8F7,$D8F7,$DAF7,$D8F7));
Const pinsel :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($003F,$003F,$0074,$0068,
                         $00F0,$01B0,$0320,$0220,
                         $0640,$04C0,$0C80,$0900,
                         $1700,$2C00,$3800,$E000));

Const hand :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0060,$06A6,$0AAA,$0AAA,
                         $6AAA,$AAAA,$AAAA,$AAAA,
                         $BAB2,$8382,$C004,$4964,
                         $2D48,$2550,$1010,$1FE0));
Const Zahnrad :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0000,$0014,$001D,$0062,
                         $0023,$055C,$2FD2,$7338,
                         $2310,$C78C,$7CF8,$C78C,
                         $2310,$7338,$2FD0,$0540));
Const apfelmann :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0380,$0440,$0440,$0440,
                         $0280,$0C60,$3018,$600C,
                         $C006,$8002,$8002,$C006,
                         $4004,$600C,$3838,$07C0));
Const mann :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($07F8,$1DAC,$3B7E,$6B77,
                         $EB35,$8921,$8001,$9CB9,
                         $8CB1,$8081,$C083,$40C2,
                         $6212,$39E6,$0C0C,$07F8));
Const Note :
      shape = (screen : ($FF87,$FF81,$FF80,$FF80,
                         $FF8C,$FF8F,$FF8F,$F00F,
                         $C00F,$800F,$8F8F,$8F8F,
                         $8F8F,$801F,$C03F,$F07F);
               mask   : ($0078,$0046,$0051,$005D,
                         $0053,$0050,$0050,$0FD0,
                         $3050,$4F90,$5050,$5050,
                         $5050,$4FA0,$3040,$0F80));
Const Noten :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($FFFF,$0000,$1C00,$FFFF,
                         $1380,$1004,$FFFF,$1004,
                         $1004,$FFFF,$7004,$7004,
                         $FFFF,$001C,$001C,$003E));
Const Violinschlussel :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0100,$0280,$0280,$0280,
                         $0300,$0600,$0A00,$1780,
                         $2A40,$2240,$2240,$1F80,
                         $0600,$0200,$1200,$0C00));
Const Mannchen :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($1FB4,$732C,$4082,$5E7D,
                         $5555,$9145,$9B2D,$85D1,
                         $8142,$81C2,$B812,$9864,
                         $87C4,$4008,$3008,$0FF0));
Const Baum :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0FF0,$180E,$3621,$6C31,
                         $4811,$8819,$8C02,$8422,
                         $8032,$7C12,$07F2,$3784,
                         $0E80,$0F80,$0F80,$1FE0));
Const Fadenkreuz :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0100,$4102,$2104,$17E8,
                         $1918,$2524,$2344,$FFFF,
                         $2184,$2344,$1528,$0930,
                         $17C8,$2104,$4102,$0100));
Const Kleinkreuz :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0400,$0400,$0400,$0000,
                         $F5E0,$0000,$0400,$0400,
                         $0400,$0000,$0000,$0000,
                         $0000,$0000,$0000,$0000));
Const Kreuz :
      shape = (screen : ($FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF,
                         $FFFF,$FFFF,$FFFF,$FFFF);
               mask   : ($0180,$0180,$0180,$0180,
                         $0180,$0180,$0180,$FFFF,
                         $FFFF,$0180,$0180,$0180,
                         $0180,$0180,$0180,$0180));


{ !!! }

procedure mausinit(var vorhanden:boolean;var tasten:word);
procedure maussicht;
procedure mausunsicht;
procedure maustastpos(var tasten,xpos,ypos:word);
procedure maussetpos(x,y:word);
procedure mausgetstatus(var x,y,buttons:integer;textmodus:boolean);
procedure mausbuttonpressinfo(var taste,x,y:word);
procedure mausbuttonreleaseinfo(var taste,x,y:word);
procedure mausminmaxx(minimum,maximum:word);
procedure mausminmaxy(minimum,maximum:word);
{ !!!, siehe oben }
procedure mausgrafikcursor(relx,rely:word;zeiger:shape);
procedure mausgrafikhercules(relx,rely:word;zeiger:shape;pathtodriver:string);
{ !!! }
procedure maustextcursor(hartsoft,maske_end,wert_start:word);
procedure mausbewegung(var xzahler,yzahler:word);
procedure mausinterrupt(maske:word;einsprung:pointer);
procedure mauslightpenon;
procedure mauslightpenoff;
procedure mausgeschwindigkeit(xrate,yrate:word);
procedure mausbedingtesaus(links,oben,rechts,unten:word);
procedure mausdoppelt(abwann:word);
procedure mausinterruptaustausch(var maske:word;neu:pointer);
procedure maussetseite(seite:word);
procedure mausgetseite(seite:word);
function mauskeypressed:boolean;
function mausinxy(x1,y1,x2,y2:integer;textmodus:boolean):boolean;

implementation

USES Dos,Graph;

var mausreg:registers;

procedure mausinit;
var
  vorh : integer;
begin
  with mausreg do begin
    ax:=0;
    intr($33,mausreg);
    vorh:=integer(ax);
    if (vorh = -1) then
      vorhanden := true
    else vorhanden := false;
    tasten:=integer(bx);
  end;
end;

procedure maussicht;
begin
  with mausreg do begin
    ax:=1;
    intr($33,mausreg);
  end;
end;

procedure mausunsicht;
begin
  with mausreg do begin
    ax:=2;
    intr($33,mausreg);
  end;
end;

procedure maustastpos(var tasten,xpos,ypos:word);
begin
  with mausreg do begin
    ax:=3;
    intr($33,mausreg);
    tasten:=bx;
    xpos:=cx;
    ypos:=dx;
  end;
end;

procedure maussetpos(x,y:word);
begin
  with mausreg do begin
    ax:=4;
    cx:=x;
    dx:=y;
    intr($33,mausreg);
  end;
end;

procedure mausbuttonpressinfo(var taste,x,y:word);
begin
  with mausreg do begin
    ax:=5;
    bx:=taste;
    intr($33,mausreg);
    taste:=bx;
    x:=cx;
    y:=dx;
  end;
end;

procedure mausbuttonreleaseinfo(var taste,x,y:word);
begin
  with mausreg do begin
    ax:=6;
    bx:=taste;
    intr($33,mausreg);
    taste:=bx;
    x:=cx;
    y:=dx;
  end;
end;

procedure mausminmaxx(minimum,maximum:word);
begin
  with mausreg do begin
    ax:=7;
    cx:=minimum;
    dx:=maximum;
    intr($33,mausreg);
  end;
end;

procedure mausminmaxy(minimum,maximum:word);
begin
  with mausreg do begin
    ax:=8;
    cx:=minimum;
    dx:=maximum;
    intr($33,mausreg);
  end;
end;

procedure mausgrafikcursor(relx,rely:word;zeiger:shape);
begin
  with mausreg do begin
    ax:=9;
    bx:=relx;
    cx:=rely;
    es:=seg(zeiger);
    dx:=ofs(zeiger);
    intr($33,mausreg);
  end;
end;

procedure mausgrafikhercules;
var
  gd,gm : integer;
  tasten : word;
  vorhanden : boolean;
begin
  gd := hercmono;
  gm := hercmonohi;
  initgraph(gd,gm,pathtodriver);
  if graphresult <> 0 then exit;
  mem[$40:$49] := 6;
  mausinit(vorhanden,tasten);
  if not vorhanden then exit;
  with mausreg do begin
    ax:=9;
    bx:=relx;
    cx:=rely;
    es:=seg(zeiger);
    dx:=ofs(zeiger);
    intr($33,mausreg);
  end;
end;

procedure maustextcursor(hartsoft,maske_end,wert_start:word);
begin
  with mausreg do begin
    ax:=10;
    bx:=hartsoft;
    cx:=maske_end;
    dx:=wert_start;
    intr($33,mausreg);
  end;
end;

procedure mausbewegung(var xzahler,yzahler:word);
begin
  with mausreg do begin
    ax:=11;
    intr($33,mausreg);
    xzahler:=cx;
    yzahler:=dx;
  end;
end;

procedure mausinterrupt(maske:word;einsprung:pointer);
begin
  with mausreg do begin
    ax:=12;
    cx:=maske;
    es:=seg(einsprung);
    dx:=ofs(einsprung);
    intr($33,mausreg);
  end;
end;

procedure mauslightpenon;
begin
  with mausreg do begin
    ax:=13;
    intr($33,mausreg);
  end;
end;

procedure mauslightpenoff;
begin
  with mausreg do begin
    ax:=14;
    intr($33,mausreg);
  end;
end;

procedure mausgeschwindigkeit(xrate,yrate:word);
begin
  with mausreg do begin
    ax:=15;
    cx:=xrate;
    dx:=yrate;
    intr($33,mausreg);
  end;
end;

procedure mausbedingtesaus(links,oben,rechts,unten:word);
begin
  with mausreg do begin
    ax:=16;
    cx:=links;
    dx:=oben;
    si:=rechts;
    di:=unten;
    intr($33,mausreg);
  end;
end;

procedure mausdoppelt(abwann:word);
begin
  with mausreg do begin
    ax:=19;
    dx:=abwann;
    intr($33,mausreg);
  end;
end;

procedure mausinterruptaustausch(var maske:word;neu:pointer);
type point=pointer;
var alt: ARRAY[1..2] OF WORD;
begin
  with mausreg do begin
    ax:=20;
    cx:=maske;
    bx:=seg(neu);
    dx:=ofs(neu);
    intr($33,mausreg);
    maske:=cx;
    alt[1]:=bx;
    alt[2]:=dx;
    neu:=point(alt);
  end;
end;

procedure maussetseite(seite:word);
begin
  with mausreg do begin
    ax:=29;
    bx:=seite;
    intr($33,mausreg);
  end;
end;

procedure mausgetseite(seite:word);
begin
  with mausreg do begin
    ax:=30;
    intr($33,mausreg);
    seite:=bx;
  end;
end;

procedure mausgetstatus;
begin
  with mausreg do begin
    ax := 3;
    intr($33,mausreg);
    x := cx;
    y := dx;
    buttons := bx;
  end;
  if textmodus then begin
    x := succ(x div 8);
    y := succ(y div 8);
  end;
end;

function mauskeypressed:boolean;
var tasten,xpos,ypos:word;
begin
  maustastpos(tasten,xpos,ypos);
  if tasten>0 then mauskeypressed:=true
  else mauskeypressed:=false;
end;

function mausinxy;
var
  temp,x,y : integer;
begin
  if x1 > x2 then begin
    temp := x1;
    x1 := x2;
    x2 := temp;
   end;
  if y1 > y2 then begin
    temp := y1;
    y1 := y2;
    y2 := temp;
  end;
  mausgetstatus(x,y,temp,textmodus);
  mausinxy := (x >= x1) and (x <= x2) and (y >= y1) and (y <= y2);
end;

end.